@@ -16,15 +16,17 @@ class HistoricalMarketDataClient(MetaApiClient):
             domain: Domain to connect to, default is agiliumtrade.agiliumtrade.ai.
         """
         super().__init__(http_client, token, domain)
-        self._host = f'https://mt-market-data-client-api-v1.{domain}'
+        self._domain = domain
+        self._urlCache = None
 
-    async def get_historical_candles(self, account_id: str, symbol: str, timeframe: str, start_time: datetime = None,
-                                     limit: int = None) -> List[MetatraderCandle]:
+    async def get_historical_candles(self, account_id: str, region: str, symbol: str, timeframe: str,
+                                     start_time: datetime = None, limit: int = None) -> List[MetatraderCandle]:
         """Returns historical candles for a specific symbol and timeframe from a MetaTrader account.
         See https://metaapi.cloud/docs/client/restApi/api/retrieveMarketData/readHistoricalCandles/
 
         Args:
             account_id: MetaTrader account id.
+            region: Account region.
             symbol: Symbol to retrieve candles for (e.g. a currency pair or an index).
             timeframe: Defines the timeframe according to which the candles must be generated. Allowed values
             for MT5 are 1m, 2m, 3m, 4m, 5m, 6m, 10m, 12m, 15m, 20m, 30m, 1h, 2h, 3h, 4h, 6h, 8h, 12h, 1d, 1w, 1mn.
@@ -38,6 +40,7 @@ class HistoricalMarketDataClient(MetaApiClient):
         """
 
         symbol = parse.quote(symbol)
+        host = await self._get_host(region)
         qs = {}
         if start_time is not None:
             qs['startTime'] = format_date(start_time)
@@ -45,7 +48,7 @@ class HistoricalMarketDataClient(MetaApiClient):
             qs['limit'] = limit
 
         opts = {
-            'url': f'{self._host}/users/current/accounts/{account_id}/historical-market-data/symbols/'
+            'url': f'{host}/users/current/accounts/{account_id}/historical-market-data/symbols/'
                    f'{symbol}/timeframes/{timeframe}/candles',
             'method': 'GET',
             'headers': {
@@ -59,13 +62,14 @@ class HistoricalMarketDataClient(MetaApiClient):
             c['time'] = date(c['time'])
         return candles
 
-    async def get_historical_ticks(self, account_id: str, symbol: str, start_time: datetime = None, offset: int = None,
-                                   limit: int = None) -> List[MetatraderTick]:
+    async def get_historical_ticks(self, account_id: str, region: str, symbol: str, start_time: datetime = None,
+                                   offset: int = None, limit: int = None) -> List[MetatraderTick]:
         """Returns historical ticks for a specific symbol from a MetaTrader account.
         See https://metaapi.cloud/docs/client/restApi/api/retrieveMarketData/readHistoricalTicks/
 
         Args:
             account_id: MetaTrader account id.
+            region: Account region.
             symbol: Symbol to retrieve ticks for (e.g. a currency pair or an index).
             start_time: Time to start loading ticks from. Note that ticks are loaded in backwards direction, so
             this should be the latest time. Leave empty to request latest ticks.
@@ -77,6 +81,7 @@ class HistoricalMarketDataClient(MetaApiClient):
         """
 
         symbol = parse.quote(symbol)
+        host = await self._get_host(region)
         qs = {}
         if start_time is not None:
             qs['startTime'] = format_date(start_time)
@@ -86,7 +91,7 @@ class HistoricalMarketDataClient(MetaApiClient):
             qs['limit'] = limit
 
         opts = {
-            'url': f'{self._host}/users/current/accounts/{account_id}/historical-market-data/symbols/{symbol}/ticks',
+            'url': f'{host}/users/current/accounts/{account_id}/historical-market-data/symbols/{symbol}/ticks',
             'method': 'GET',
             'headers': {
                 'auth-token': self._token
@@ -98,3 +103,24 @@ class HistoricalMarketDataClient(MetaApiClient):
         for t in ticks:
             t['time'] = date(t['time'])
         return ticks
+
+    async def _get_host(self, region: str):
+        if self._urlCache and self._urlCache['lastUpdated'] > datetime.now().timestamp() - 600:
+            return self._urlCache['url']
+
+        url_settings = await self._httpClient.request({
+            'url': f'https://mt-provisioning-api-v1.{self._domain}/users/current/servers/mt-client-api',
+            'method': 'GET',
+            'headers': {
+                'auth-token': self._token
+            }
+        })
+
+        url = f'https://mt-market-data-client-api-v1.{region}.{url_settings["domain"]}'
+
+        self._urlCache = {
+            'url': url,
+            'lastUpdated': datetime.now().timestamp()
+        }
+
+        return url
